package com.example.controller;

import com.example.config.annotation.IgnoreAuth;
import com.example.entity.WzLowestUnitPriceEntity;
import com.example.service.WzLowestUnitPriceService;
import com.example.utils.PageMap;
import com.example.utils.PageUtils;
import com.example.utils.Query;
import com.example.utils.R;
import com.example.utils.excel.ExcelExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 房屋出租(合同管理)
 */
@RestController
@RequestMapping("/house-car/house/dist/api/wzLowestUnitPrice")
public class WzLowestUnitPriceController {

    @Autowired
    private WzLowestUnitPriceService wzLowestUnitPriceService;

    @RequestMapping("/list")
    @IgnoreAuth
    public R list(@RequestBody PageMap pageMap) {
        //查询列表数据
        Query query = new Query(pageMap);
        List<WzLowestUnitPriceEntity> resultList = wzLowestUnitPriceService.queryList(query);
        for (WzLowestUnitPriceEntity wzLowestUnitPriceEntity : resultList) {
            double price=Double.valueOf(wzLowestUnitPriceEntity.getIncludingValueAddedTaxContractAmount())/(Double.valueOf(wzLowestUnitPriceEntity.getContractLeaseTerm())*Double.valueOf(wzLowestUnitPriceEntity.getContractedRentalArea()));
            wzLowestUnitPriceEntity.setPrice(formatDouble(price));
        }
        Query query1 = new Query(pageMap);
        int total = wzLowestUnitPriceService.queryTotal(query1);

        List<WzLowestUnitPriceEntity> newList = resultList.stream().sorted(Comparator.comparing(WzLowestUnitPriceEntity::getPrice)).collect(Collectors.toList());


        Collections.reverse(newList);//反转顺序

        PageUtils pageUtil = new PageUtils(newList, total, query.getLimit(), query.getPage());
        return R.ok().put("page", pageUtil);
    }

    /**
     * 清单excel
     */
    @GetMapping("/excel")
    @IgnoreAuth
    public void excel(String name, String buildingNameAlias,String statisticalMonth, String city, String county, HttpServletResponse response) {
        Map<String, Object> map = new HashMap<>();
        if (buildingNameAlias != null) {
            map.put("buildingNameAlias", buildingNameAlias);
        }
        if (city != null) {
            map.put("city", city);
        }
        if (county != null) {
            map.put("county", county);
        }
        if(statisticalMonth!=null){
            map.put("statisticalMonth", statisticalMonth);
        }
        List<WzLowestUnitPriceEntity> resultList = wzLowestUnitPriceService.queryList(map);
        for (WzLowestUnitPriceEntity wzLowestUnitPriceEntity : resultList) {
            double price=Double.valueOf(wzLowestUnitPriceEntity.getIncludingValueAddedTaxContractAmount())/(Double.valueOf(wzLowestUnitPriceEntity.getContractLeaseTerm())*Double.valueOf(wzLowestUnitPriceEntity.getContractedRentalArea()));
            wzLowestUnitPriceEntity.setPrice(formatDouble(price));
        }
        resultList = resultList.stream().sorted(Comparator.comparing(WzLowestUnitPriceEntity::getPrice)).collect(Collectors.toList());
        Collections.reverse(resultList);//反转顺序

        DateFormat df = new SimpleDateFormat("yyyyMMdd");
        ExcelExport ee1 = new ExcelExport("出租监控_每类地段每平米出租单价最低TOP10" + df.format(new Date()));

        List<Object> colList;
        List<Object[]> allList = new ArrayList();
        for (int i = 0; i < resultList.size(); i++) {
            colList = new ArrayList();
            colList.add(resultList.get(i).getCity());
            colList.add(resultList.get(i).getAreaSector());
            colList.add(resultList.get(i).getBuildingAlias());
            colList.add(resultList.get(i).getRentalStartDate());
            colList.add(resultList.get(i).getContractLeaseTerm());
            colList.add(WzHouseMaintenaCostController.toThousands(resultList.get(i).getContractedRentalArea(),true));
            colList.add(WzHouseMaintenaCostController.toThousands(resultList.get(i).getIncludingValueAddedTaxContractAmount(),true));
            colList.add(WzHouseMaintenaCostController.toThousands(resultList.get(i).getPrice(),true));
            colList.add(resultList.get(i).getContractName());
            colList.add(resultList.get(i).getUserName());
            colList.add(resultList.get(i).getUserType());
            colList.add(resultList.get(i).getAddress());
            allList.add(colList.toArray());
        }
        String[] header = new String[]{
                "二级单位",
                "地段",
                "建筑/土地别名",
                "出租起始日期",
                "合同租期（月缴）",
                "合同出租面积",
                "含增值税合同金额",
                "每平米单价=不含税总价/租期/面积",
                "合同名称",
                "对方全称",
                "客户类型",
                "标准地址",

        };
        ee1.addSheetByArray(df.format(new Date()) + "清单", allList, header);
        ee1.export(response);
    }

    public double formatDouble(double num){
        double avera=0;
        try{
            DecimalFormat format2 = new DecimalFormat("#.00");
            String str2 = format2.format(num);
            avera = Double.parseDouble(str2);
        }catch (Exception e){
            e.printStackTrace();
        }
        return avera;
    }
}
