package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.dao.house.HouseRepairExceptionDao;
import com.example.entity.house.HouseBuildingHighRepairPo;
import com.example.enums.OrderEnum;
import com.example.pojo.dto.ListBuildingHighRepairDto;
import com.example.pojo.bo.ListBuildingHighRepairBo;
import com.example.pojo.vo.ListBuildingHighRepairVo;
import com.example.utils.PageUtils;
import com.example.utils.R;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Slf4j
@Service
public class HouseWzHouseMaintenaCostService {
    private final HouseRepairExceptionDao houseRepairExceptionDao;

    public HouseWzHouseMaintenaCostService(HouseRepairExceptionDao houseRepairExceptionDao) {
        this.houseRepairExceptionDao = houseRepairExceptionDao;
    }

    public R queryHighPrice(ListBuildingHighRepairDto dto) {
        Page<HouseBuildingHighRepairPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<HouseBuildingHighRepairPo> list = getHouseBuildingHighRepairPos(dto, page);
        List<ListBuildingHighRepairVo> vos = new ArrayList<>();
        for (HouseBuildingHighRepairPo po : list) {
            vos.add(new ListBuildingHighRepairVo(po));
        }
        PageUtils pageUtil = new PageUtils(vos, page);
        return R.ok().put("page", pageUtil);
    }

    public void queryHighPriceExport(ListBuildingHighRepairDto dto) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder
                .getRequestAttributes();
        if (servletRequestAttributes == null) {
            return;
        }
        HttpServletResponse response = servletRequestAttributes.getResponse();
        if (response == null) {
            return;
        }
        String filename = "维修异常监控_高总价统计"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeQueryHighPrice(dto, wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private List<HouseBuildingHighRepairPo> getHouseBuildingHighRepairPos(ListBuildingHighRepairDto dto,
                                                                          Page<HouseBuildingHighRepairPo> page) {
        ListBuildingHighRepairBo bo = new ListBuildingHighRepairBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getSidx() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getSidx().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getSidx().getColumnName()));
            }
        }
        return houseRepairExceptionDao.listBuildingHighRepair(page, bo);
    }

    /**
     * 写高总价统计文件
     */
    private void writeQueryHighPrice(ListBuildingHighRepairDto dto, SXSSFWorkbook wb) {
        Page<HouseBuildingHighRepairPo> page = new Page<>(1, Long.MAX_VALUE);
        List<HouseBuildingHighRepairPo> list = getHouseBuildingHighRepairPos(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger columnIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(columnIndex.getAndIncrement());
        List<String> headers = Stream.of("入账日期", "二级单位", "三级单位", "建筑别名", "维修类型", "维修内容",
                "erp入账金额元", "标准地址", "建筑用途", "建筑面积㎡", "建筑面积自用㎡", "建筑面积出租㎡", "建筑面积闲置㎡",
                        "建筑面积不可使用㎡").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (HouseBuildingHighRepairPo houseBuildingHighRepairPo : list) {
            Row row = sheet.createRow(columnIndex.getAndIncrement());
            // 入账日期
            Cell a = row.createCell(CellReference.convertColStringToIndex("a"));
            if (houseBuildingHighRepairPo.getLatestRepairMonth() != null) {
                a.setCellValue(houseBuildingHighRepairPo.getLatestRepairMonth());
            }
            // 二级单位
            Cell b = row.createCell(CellReference.convertColStringToIndex("b"));
            if (houseBuildingHighRepairPo.getAreaName() != null) {
                b.setCellValue(houseBuildingHighRepairPo.getAreaName());
            }
            // 三级单位
            Cell c = row.createCell(CellReference.convertColStringToIndex("c"));
            if (houseBuildingHighRepairPo.getCityName() != null) {
                c.setCellValue(houseBuildingHighRepairPo.getCityName());
            }
            // 建筑别名
            Cell d = row.createCell(CellReference.convertColStringToIndex("d"));
            if (houseBuildingHighRepairPo.getBuildingName() != null) {
                d.setCellValue(houseBuildingHighRepairPo.getBuildingName());
            }
            // 维修类型
            Cell e = row.createCell(CellReference.convertColStringToIndex("e"));
            if (houseBuildingHighRepairPo.getRepairType() != null) {
                e.setCellValue(houseBuildingHighRepairPo.getRepairType());
            }
            // 维修内容
            Cell f = row.createCell(CellReference.convertColStringToIndex("f"));
            if (houseBuildingHighRepairPo.getContentCategory() != null) {
                f.setCellValue(houseBuildingHighRepairPo.getContentCategory());
            }
            // erp入账金额元
            Cell g = row.createCell(CellReference.convertColStringToIndex("g"));
            if (houseBuildingHighRepairPo.getFinalCostSum() != null) {
                g.setCellValue(houseBuildingHighRepairPo.getFinalCostSum().doubleValue());
            }
            g.setCellStyle(numberCellStyle);
            // 标准地址
            Cell h = row.createCell(CellReference.convertColStringToIndex("h"));
            if (houseBuildingHighRepairPo.getAddress() != null) {
                h.setCellValue(houseBuildingHighRepairPo.getAddress());
            }
            // 建筑用途
            Cell i = row.createCell(CellReference.convertColStringToIndex("i"));
            if (houseBuildingHighRepairPo.getBuildingUse() != null) {
                i.setCellValue(houseBuildingHighRepairPo.getBuildingUse());
            }
            // 建筑面积㎡
            Cell j = row.createCell(CellReference.convertColStringToIndex("j"));
            if (houseBuildingHighRepairPo.getBuildingArea() != null) {
                j.setCellValue(houseBuildingHighRepairPo.getBuildingArea().doubleValue());
            }
            j.setCellStyle(numberCellStyle);
            // 建筑面积自用㎡
            Cell k = row.createCell(CellReference.convertColStringToIndex("k"));
            if (houseBuildingHighRepairPo.getBuildingAreaSelfUse() != null) {
                k.setCellValue(houseBuildingHighRepairPo.getBuildingAreaSelfUse().doubleValue());
            }
            k.setCellStyle(numberCellStyle);
            // 建筑面积出租㎡
            Cell l = row.createCell(CellReference.convertColStringToIndex("l"));
            if (houseBuildingHighRepairPo.getBuildingAreaRent() != null) {
                l.setCellValue(houseBuildingHighRepairPo.getBuildingAreaRent().doubleValue());
            }
            l.setCellStyle(numberCellStyle);
            // 建筑面积闲置㎡
            Cell m = row.createCell(CellReference.convertColStringToIndex("m"));
            if (houseBuildingHighRepairPo.getBuildingAreaIdle() != null) {
                m.setCellValue(houseBuildingHighRepairPo.getBuildingAreaIdle().doubleValue());
            }
            m.setCellStyle(numberCellStyle);
            // 建筑面积不可使用㎡
            Cell n = row.createCell(CellReference.convertColStringToIndex("n"));
            if (houseBuildingHighRepairPo.getBuildingAreaUnusable() != null) {
                n.setCellValue(houseBuildingHighRepairPo.getBuildingAreaUnusable().doubleValue());
            }
            n.setCellStyle(numberCellStyle);
        }
    }
}
